'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var immutable = require('immutable');
var isPlainObject = _interopDefault(require('is-plain-object'));
var slate = require('slate');
var slateKitPluginsUtils = require('@vericus/slate-kit-plugins-utils');
var tinycolor = _interopDefault(require('tinycolor2'));
var Renderer = _interopDefault(require('@vericus/slate-kit-highlight-text-renderer'));

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var defaultOptions = {
  type: undefined,
  styles: undefined,
  data: undefined,
  defaultColor: undefined,
  alpha: 1,
  externalRenderer: false
};

var Options = function (_Record) {
  inherits(Options, _Record);

  function Options() {
    classCallCheck(this, Options);
    return possibleConstructorReturn(this, (Options.__proto__ || Object.getPrototypeOf(Options)).apply(this, arguments));
  }

  createClass(Options, [{
    key: "toJS",


    /**
     * Alias `toJS`.
     */

    value: function toJS(options) {
      return this.toJSON(options);
    }
  }], [{
    key: "create",
    value: function create() {
      var attrs = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      if (Options.isOptions(attrs)) return attrs;
      if (isPlainObject(attrs)) return Options.fromJSON(attrs);

      throw new Error("`Options.create` only accepts objects, strings or options");
    }
  }, {
    key: "fromJSON",
    value: function fromJSON(object) {
      if (Options.isOptions(object)) return object;

      var type = object.type,
          data = object.data,
          styles = object.styles,
          defaultColor = object.defaultColor,
          _object$alpha = object.alpha,
          alpha = _object$alpha === undefined ? 1 : _object$alpha;

      if (!type || !data || !styles || !defaultColor || !Array.isArray(styles)) {
        throw new Error("Highlight text require type, data and style option");
      }

      if (alpha < 0 || alpha > 1) {
        throw new Error("Alpha value can only go from 0 to 1");
      }

      return new Options({
        type: type,
        data: data,
        styles: styles,
        alpha: alpha,
        defaultColor: defaultColor
      });
    }
  }, {
    key: "isOptions",
    value: function isOptions(any) {
      return !!(any instanceof immutable.Record && ["type", "data", "styles", "defaultColor"].every(function (key) {
        return any.has(key);
      }));
    }
  }]);
  return Options;
}(immutable.Record(defaultOptions, "highlight text option"));

function createChanges(opt) {
  var defaultColor = opt.defaultColor,
      data = opt.data,
      type = opt.type;

  return {
    changeColor: function changeColor(change, color) {
      var value = change.value;
      var selection = value.selection,
          document = value.document;

      if (slateKitPluginsUtils.hasMark(value, type)) {
        if (tinycolor(color).toName() === defaultColor) {
          if (selection.isCollapsed) {
            slateKitPluginsUtils.removeCollapsedMark(value, change, type);
          } else {
            slateKitPluginsUtils.removeExpandedMark(document, selection, change, type);
          }
        } else {
          var newMark = new slate.Mark({
            type: type,
            data: slate.Data.create(defineProperty({}, data, color))
          });
          if (selection.isCollapsed) {
            slateKitPluginsUtils.removeCollapsedMark(value, change, type);
            change.focus();
            change.addMark(newMark);
          } else {
            slateKitPluginsUtils.removeExpandedMark(document, selection, change, type);
            change.addMarkAtRange(selection, newMark);
          }
        }
      } else if (tinycolor(color).toName() !== defaultColor) {
        var mark = new slate.Mark({
          type: type,
          data: slate.Data.create(defineProperty({}, data, color))
        });
        if (selection.isCollapsed) {
          change.addMark(mark);
        } else {
          slateKitPluginsUtils.removeExpandedMark(document, selection, change, type);
          change.addMarkAtRange(selection, mark);
        }
      }
      change.focus();
      return change;
    }
  };
}

function createUtils(opts) {
  var defaultColor = opts.defaultColor,
      data = opts.data,
      type = opts.type;

  return {
    currentColor: function currentColor(value) {
      if (value.selection && value.selection.isFocused && value.activeMarks) {
        var activeColorMarks = value.activeMarks.filter(function (mark) {
          return mark.type === type;
        }).map(function (mark) {
          return mark.data.get(data);
        });
        var uniqueColorMarks = [].concat(toConsumableArray(new Set(activeColorMarks)));
        if (uniqueColorMarks.length !== 0) {
          return uniqueColorMarks[0];
        }
      }
      return defaultColor;
    }
  };
}

function createSchema() {
  var schemas = {};
  var schema = {};
  schemas.getSchema = function () {
    return schema;
  };
  return schemas;
}

function createPlugin(pluginOptions) {
  var options = Options.create(pluginOptions);
  var changes = createChanges(options);
  var schemas = createSchema();
  var utils = createUtils(options);
  var plugin = _extends({
    options: options,
    changes: changes,
    utils: utils
  }, schemas);
  if (!options.externalRenderer) {
    var _Renderer = Renderer(options),
        renderMark = _Renderer.renderMark;

    plugin.renderMark = renderMark;
  }
  return plugin;
}

exports.default = createPlugin;
//# sourceMappingURL=slate-kit-highlight-text.js.map
